package com.vasilz.nytnews;

import java.io.File;
import java.io.IOException;
import java.util.List;

import org.apache.lucene.analysis.SimpleAnalyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.document.Field.Index;
import org.apache.lucene.document.Field.Store;
import org.apache.lucene.document.Fieldable;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.IndexWriterConfig;
import org.apache.lucene.store.FSDirectory;
import org.apache.lucene.store.LockObtainFailedException;
import org.apache.lucene.util.Version;

public class Indexer {
	
private final File indexDirectory;
	
	public Indexer(File indexDir) {
		indexDirectory = indexDir;
	}
	
	public void index(List<News> newsList) throws CorruptIndexException, LockObtainFailedException, IOException {
		IndexWriter writer = new IndexWriter(FSDirectory.open(indexDirectory), 
				new IndexWriterConfig(Version.LUCENE_32, new StandardAnalyzer(Version.LUCENE_32)));
		for (News news : newsList) {
			System.out.println("Adding news...:" + news);
			Document newsDoc = new Document();
			newsDoc.add(new Field(News.LUCENE_KEY_SECTION, news.getSection(), Store.YES, Index.NO));
			newsDoc.add(new Field(News.LUCENE_KEY_TITLE, news.getTitle(), Store.YES, Index.NO));
			newsDoc.add(new Field(News.LUCENE_KEY_ABSTRACT, news.getNewsAbstract(), Store.YES, Index.NO));
			newsDoc.add(new Field(News.LUCENE_KEY_URL, news.getUrl(), Store.YES, Index.NO));
			newsDoc.add(new Field(News.LUCENE_KEY_BYLINE, news.getByline(), Store.YES, Index.NO));
			newsDoc.add(new Field(News.LUCENE_KEY_ITEMTYPE, news.getItemType(), Store.YES, Index.NO));
			newsDoc.add(new Field(News.LUCENE_KEY_SOURCE, news.getSource(), Store.YES, Index.NO));
			newsDoc.add(new Field(News.LUCENE_KEY_PUBLISHEDON, news.getPublishedOn().toString(), Store.YES, Index.NO));
			newsDoc.add(new Field(News.LUCENE_KEY_DESFACET, news.getDesFacetString().toLowerCase(), Store.YES, Index.ANALYZED));
			newsDoc.add(new Field(News.LUCENE_KEY_GEOFACET, news.getGeoFacetString().toLowerCase(), Store.YES, Index.ANALYZED));
			writer.addDocument(newsDoc);
		}
		writer.commit();
		writer.close();
	}

}
