package com.vasilz.nytnews;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.codehaus.jackson.annotate.JsonProperty;

public class News {
	
	public static final String LUCENE_KEY_SECTION = "section";
	public static final String LUCENE_KEY_TITLE = "title";
	public static final String LUCENE_KEY_ABSTRACT = "abstract";
	public static final String LUCENE_KEY_URL = "url";
	public static final String LUCENE_KEY_BYLINE = "byline";
	public static final String LUCENE_KEY_ITEMTYPE = "item_type";
	public static final String LUCENE_KEY_SOURCE = "source";
	public static final String LUCENE_KEY_PUBLISHEDON = "published_date";
	public static final String LUCENE_KEY_DESFACET = "des_facet";
	public static final String LUCENE_KEY_GEOFACET = "geo_facet";
	
	@JsonProperty("section")
	private String section;
	@JsonProperty("title")
	private String title;
	@JsonProperty("abstract")
	private String newsAbstract;
	@JsonProperty("url")
	private String url;
	@JsonProperty("byline")
	private String byline;
	@JsonProperty("item_type")
	private String itemType; //ENUM?
	@JsonProperty("source")
	private String source; //ENUM?
	@JsonProperty("published_date")
	private Date publishedOn;
	
	private List<String> desFacet;
	
	private List<String> geoFacet;
	
	public News() {
		desFacet = new ArrayList<String>();
		geoFacet = new ArrayList<String>();
	}
	
	public News(String section, String title, String newsAbstract, String url,
			String byline, String itemType, String source, Date publishedOn, 
			List<String> des, List<String> geo) {
		this.section = section;
		this.title = title;
		this.newsAbstract = newsAbstract;
		this.url = url;
		this.byline = byline;
		this.itemType = itemType;
		this.source = source;
		this.publishedOn = publishedOn;
		desFacet = new ArrayList<String>();
		if (des != null) {
			desFacet.addAll(des);
		}
		geoFacet = new ArrayList<String>();
		if (geo != null) {
			geoFacet.addAll(geo);
		}
	}

	public void setSection(String section) {
		this.section = section;
	}

	public String getSection() {
		return section;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public String getTitle() {
		return title;
	}

	public void setNewsAbstract(String newsAbstract) {
		this.newsAbstract = newsAbstract;
	}

	public String getNewsAbstract() {
		return newsAbstract;
	}

	public void setUrl(String url) {
		this.url = url;
	}

	public String getUrl() {
		return url;
	}

	public void setByline(String byline) {
		this.byline = byline;
	}

	public String getByline() {
		return byline;
	}

	public void setItemType(String itemType) {
		this.itemType = itemType;
	}

	public String getItemType() {
		return itemType;
	}

	public void setSource(String source) {
		this.source = source;
	}

	public String getSource() {
		return source;
	}

	public void setPublishedOn(Date publishedOn) {
		this.publishedOn = publishedOn;
	}

	public Date getPublishedOn() {
		return publishedOn;
	}

	public List<String> getDesFacet() {
		return desFacet;
	}
	
	public String getDesFacetString() {
		if (desFacet.isEmpty()) {
			return "";
		}
		StringBuffer buf = new StringBuffer();
		for (String des : desFacet) {
			buf.append(des).append(" ");
		}
		buf.deleteCharAt(buf.length()-1);
		return buf.toString();
	}
	
	public String getGeoFacetString() {
		if (geoFacet.isEmpty()) {
			return "";
		}
		StringBuffer buf = new StringBuffer();
		for (String geo : geoFacet) {
			buf.append(geo).append(" ");
		}
		buf.deleteCharAt(buf.length()-1);
		return buf.toString();
	}
	
	public List<String> getGeoFacet() {
		return geoFacet;
	}
	
	@JsonProperty("des_facet")
	public void setDesFacet(Object des) {
		if (des == null) {
			return;
		}
		if (des instanceof List<?>) {
			desFacet.addAll((List<String>)des);
		}
	}
	
	@JsonProperty("geo_facet")
	public void setGeoFacet(Object geo) {
		if (geo == null) {
			return;
		}
		if (geo instanceof List<?>) {
			geoFacet.addAll((List<String>)geo);
		}
	}
	
	@Override
	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append("News:[section:").append(section);
		buf.append(",title:").append(title);
		buf.append(",abstract:").append(newsAbstract);
		buf.append(",url:").append(url);
		buf.append(",byline:").append(byline);
		buf.append(",item_type:").append(itemType);
		buf.append(",source:").append(source);
		buf.append(",published_date:").append(publishedOn);
		buf.append(",des_facet:").append(desFacet);
		buf.append(",geo_facet:").append(geoFacet);
		return buf.toString();
		
	}
	
	//TODO implemetn hashCode()
	
	@Override
	public boolean equals(Object obj) {
		if (obj instanceof News) {
			News other = (News) obj;
			if (section != other.section || section != null && !section.equals(other.section)) {
				return false;
			}
			if (title != other.title || title != null && !title.equals(other.title)) {
				return false;
			}
			if (newsAbstract != other.newsAbstract || newsAbstract != null && !newsAbstract.equals(other.newsAbstract)) {
				return false;
			}
			if (url != other.url || url != null && !url.equals(other.url)) {
				return false;
			}
			if (byline != other.byline || byline != null && !byline.equals(other.byline)) {
				return false;
			}
			if (itemType != other.itemType || itemType != null && !itemType.equals(other.itemType)) {
				return false;
			}
			if (source != other.source || source != null && !source.equals(other.source)) {
				return false;
			}
			if (publishedOn != other.publishedOn || publishedOn != null && publishedOn.compareTo(other.publishedOn) != 0) {
				return false;
			}
			return true;
		}
		return false;
	}
}
