package com.vasilz.nytnews;

import java.io.File;
import java.io.IOException;
import java.sql.Date;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.DuplicateFilter;
import org.apache.lucene.search.Filter;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.search.TopDocs;
import org.apache.lucene.store.FSDirectory;
import org.apache.lucene.util.Version;

public class Searcher {
	
	private static HashMap<String, Searcher> instances = new HashMap<String, Searcher>();
	
	private IndexReader reader;
	
	private Searcher(File indexDir) throws CorruptIndexException, IOException {
		reader = IndexReader.open(FSDirectory.open(indexDir));
	}
	
	public static Searcher get(File indexDir) throws CorruptIndexException, IOException {
		if (instances == null) {
			instances = new HashMap<String, Searcher>();
		}
		if (!instances.containsKey(indexDir.getAbsolutePath())) {
			Searcher s = new Searcher(indexDir);
			instances.put(indexDir.getAbsolutePath(), s);
		}
		return instances.get(indexDir.getAbsolutePath());
	}
	
	public List<News> searchByKeywords(List<String> keywords) throws IOException, ParseException, java.text.ParseException {
		List<News> newsResult = new ArrayList<News>();
		
		IndexSearcher is = new IndexSearcher(reader);
		QueryParser parser = new QueryParser(Version.LUCENE_31, News.LUCENE_KEY_DESFACET,
				new StandardAnalyzer(Version.LUCENE_31));
		
		DuplicateFilter df = new DuplicateFilter(News.LUCENE_KEY_TITLE);
        df.setKeepMode(DuplicateFilter.KM_USE_FIRST_OCCURRENCE);
		
		for (String keyword : keywords) {
			String queryString = News.LUCENE_KEY_DESFACET + ":" + keyword.toLowerCase() + 
						" OR " + News.LUCENE_KEY_GEOFACET + ":" + keyword.toLowerCase();
			Query query = parser.parse(queryString);
			TopDocs hits = is.search(query, NYTConnector.itemsPerRequest);
			//TopDocs hits = is.search(query, df, 100);
			ScoreDoc[] topHits = hits.scoreDocs;

			System.out.println("Results found: " + topHits.length);
			for (ScoreDoc score : topHits) {
				Document doc = is.doc(score.doc);
				News news = new News();
				news.setSection(doc.get(News.LUCENE_KEY_SECTION));
				news.setTitle(doc.get(News.LUCENE_KEY_TITLE));
				news.setNewsAbstract(doc.get(News.LUCENE_KEY_ABSTRACT));
				news.setUrl(doc.get(News.LUCENE_KEY_URL));
				news.setByline(doc.get(News.LUCENE_KEY_BYLINE));
				news.setItemType(doc.get(News.LUCENE_KEY_ITEMTYPE));
				news.setSource(doc.get(News.LUCENE_KEY_SOURCE));
				news.setPublishedOn(Util.LUCENE_DATE_FORMAT.parse(doc.get(News.LUCENE_KEY_PUBLISHEDON)));
				newsResult.add(news);
			}
		}
		
		Set<News> uniqueNews = new HashSet<News>(newsResult);
		return new ArrayList<News>(uniqueNews);
	}

}
