package com.vasilz.nytnews.gui;

import javax.swing.JFrame;
import java.awt.Dimension;
import javax.swing.JPanel;
import java.awt.GridBagLayout;
import javax.swing.BoxLayout;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;

import org.apache.lucene.index.CorruptIndexException;

import com.vasilz.nytnews.News;
import com.vasilz.nytnews.Searcher;
import com.vasilz.nytnews.Util;

import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import static com.vasilz.nytnews.Util.getNullSafeString;

public class MainFrame extends JFrame {

	private static final Object RESULTS_COLUMN_SECTION = "Section";
	private static final Object RESULTS_COLUMN_TITLE = "Title";
	private static final Object RESULTS_COLUMN_ABSTRACT = "Abstract";
	private static final Object RESULTS_COLUMN_URL = "Url";
	private static final Object RESULTS_COLUMN_BYLINE = "By-line";
	private static final Object RESULTS_COLUMN_ITEMTYPE = "Item type";
	private static final Object RESULTS_COLUMN_SOURCE = "Source";
	private static final Object RESULTS_COLUMN_PUBLISHEDON = "Published date";
	private JPanel mainPanel = null;
	private JPanel searchPanel = null;
	private JPanel resultsPanel = null;
	private JTextField keywordField = null;
	private JButton searchButton = null;
	private JScrollPane resultScroll = null;
	private JTable resultsTable = null;
	
	private DefaultTableModel resultsModel = null;
	private JButton resetButton = null;

	/**
	 * This method initializes 
	 * 
	 */
	public MainFrame() {
		super();
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
        this.setSize(new Dimension(1248, 491));
        this.setTitle("NYT News search");
        this.setContentPane(getMainPanel());
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        this.getRootPane().setDefaultButton(searchButton);
	}

	/**
	 * This method initializes mainPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getMainPanel() {
		if (mainPanel == null) {
			mainPanel = new JPanel();
			mainPanel.setLayout(new BoxLayout(getMainPanel(), BoxLayout.Y_AXIS));
			mainPanel.add(getSearchPanel(), null);
			mainPanel.add(getResultsPanel(), null);
		}
		return mainPanel;
	}

	/**
	 * This method initializes searchPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getSearchPanel() {
		if (searchPanel == null) {
			searchPanel = new JPanel();
			searchPanel.setLayout(new BoxLayout(getSearchPanel(), BoxLayout.X_AXIS));
			searchPanel.add(getKeywordField(), null);
			searchPanel.add(getSearchButton(), null);
			searchPanel.add(getResetButton(), null);
		}
		return searchPanel;
	}

	/**
	 * This method initializes resultsPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getResultsPanel() {
		if (resultsPanel == null) {
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.fill = GridBagConstraints.BOTH;
			gridBagConstraints.weighty = 1.0;
			gridBagConstraints.weightx = 1.0;
			resultsPanel = new JPanel();
			resultsPanel.setLayout(new GridBagLayout());
			resultsPanel.add(getResultScroll(), gridBagConstraints);
		}
		return resultsPanel;
	}

	/**
	 * This method initializes keywordField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getKeywordField() {
		if (keywordField == null) {
			keywordField = new JTextField();
		}
		return keywordField;
	}

	/**
	 * This method initializes searchButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getSearchButton() {
		if (searchButton == null) {
			searchButton = new JButton();
			searchButton.setText("Search");
			searchButton.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					List<String> keywords = new ArrayList<String>();
					String keywordValue = getNullSafeString(keywordField.getText());
					keywords = Arrays.asList(keywordValue.split(","));
					try {
						Searcher s = Searcher.get(Util.LUCENE_INDEX_DIR);
						List<News> newsList = s.searchByKeywords(keywords);
						populateResults(newsList);
						
					} catch (Exception e1) {
						e1.printStackTrace();
						return;
					}
					
				}
			});
		}
		return searchButton;
	}

	protected void populateResults(List<News> newsList) {
		while (resultsModel.getRowCount() > 0) {
			resultsModel.removeRow(0);
		}
		if (newsList == null) {
			JOptionPane.showMessageDialog(null, "Cannot pass null argument!", "NPE", JOptionPane.ERROR_MESSAGE);
			return;
		}
		if (newsList.isEmpty()) {
			JOptionPane.showMessageDialog(null, "No results found");
			return;
		}
		for (News news : newsList) {
			resultsModel.addRow(new Object[]{getNullSafeString(news.getSection()),
											getNullSafeString(news.getTitle()),
											getNullSafeString(news.getNewsAbstract()),
											getNullSafeString(news.getUrl()),
											getNullSafeString(news.getByline()),
											getNullSafeString(news.getItemType()),
											getNullSafeString(news.getSource()),
											getNullSafeString(news.getPublishedOn().toString())});
		}
		//resultsTable.repaint();
	}

	/**
	 * This method initializes resultScroll	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getResultScroll() {
		if (resultScroll == null) {
			resultScroll = new JScrollPane();
			resultScroll.setViewportView(getResultsTable());
		}
		return resultScroll;
	}

	/**
	 * This method initializes resultsTable	
	 * 	
	 * @return javax.swing.JTable	
	 */
	private JTable getResultsTable() {
		if (resultsTable == null) {
			resultsTable = new JTable();
			resultsModel = new DefaultTableModel(new Object[]{RESULTS_COLUMN_SECTION,
															RESULTS_COLUMN_TITLE,
															RESULTS_COLUMN_ABSTRACT,
															RESULTS_COLUMN_URL,
															RESULTS_COLUMN_BYLINE,
															RESULTS_COLUMN_ITEMTYPE,
															RESULTS_COLUMN_SOURCE,
															RESULTS_COLUMN_PUBLISHEDON}, 0);
			resultsTable.setModel(resultsModel);
		}
		return resultsTable;
	}

	/**
	 * This method initializes resetButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getResetButton() {
		if (resetButton == null) {
			resetButton = new JButton();
			resetButton.setText("Reset");
			resetButton.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					keywordField.setText("");
					while (resultsModel.getRowCount() > 0) {
						resultsModel.removeRow(0);
					}
				}
			});
		}
		return resetButton;
	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
